package com.smartechbasereactnative;

import android.app.Application;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.util.Log;

import com.netcore.android.Smartech;

import java.lang.ref.WeakReference;

public class SmartechBasePlugin implements SmartechDeeplinkReceivers.OnDeeplinkReceived {

  public static final String TAG = SmartechBasePlugin.class.getName();

  private static volatile SmartechBasePlugin instance;
  private SmartechDeeplinkCallbackHandler deeplinkCallbackHandler = null;
  private final Long maxDelayTime = 3000L;
  private final Long delayDiff = 100L;
  private final Long startDelayTime = 400L;
  private Smartech smartech = null;

  private SmartechBasePlugin() {
  }

  public static SmartechBasePlugin getInstance() {
    if (instance == null) {
      synchronized (SmartechBasePlugin.class) {
        if (instance == null) {
          instance = new SmartechBasePlugin();
        }
      }
    }
    return instance;
  }

  public void setDeeplinkHandlerListener(SmartechDeeplinkCallbackHandler deeplinkCallbackHandler) {
    this.deeplinkCallbackHandler = deeplinkCallbackHandler;
  }


  public void init(Application context) {
    try {
      Log.v(TAG,"SmartechBasePlugin - initialisation started.");
      smartech = Smartech.getInstance((new WeakReference<>(context)));
      smartech.initializeSdk(context);
      SmartechDeeplinkReceivers.setRegisterCallback(this);
      SmartechDeeplinkReceivers deeplinkReceiver = new SmartechDeeplinkReceivers();

      IntentFilter filter = new IntentFilter("com.smartech.EVENT_PN_INBOX_CLICK");
      if (Build.VERSION.SDK_INT >= 34) {
        context.registerReceiver(deeplinkReceiver, filter, Context.RECEIVER_EXPORTED);
      } else {
        context.registerReceiver(deeplinkReceiver, filter);
      }
    } catch (Exception e) {
      e.printStackTrace();
    }
  }

  public void setDebugLevel(int debugLevel) {
    if (smartech != null) {
      smartech.setDebugLevel(debugLevel);
    }
  }

  @Override
  public void onDeeplinkReceived(Intent intent) {
    try {
      Log.v(TAG,"SmartechBasePlugin - onDeeplinkReceived executed");
      if (deeplinkCallbackHandler == null) {
        Log.v(TAG,"SmartechBasePlugin - deeplink callback handler is null");
        initDeeplinkCallbackHandler(startDelayTime, intent);
      } else {
        Log.v(TAG,"SmartechBasePlugin - deeplink callback handler is available");
        deeplinkCallbackHandler.onDeeplinkIntentReceived(intent);
      }
    } catch (Exception e) {
      e.printStackTrace();
    }
  }

  /**
   * Initializes the deeplink callback handler with a specified delay and intent.
   * This method recursively tries to set up the handler if it is not available
   * initially, incrementing the delay until the maximum allowed delay is reached.
   *
   * @param delay The initial delay before attempting to initialize the handler.
   * @param intent The Intent to be sent to the deeplink callback handler once it is available.
   */
  private void initDeeplinkCallbackHandler(Long delay, Intent intent) {
    try {
      new Handler(Looper.getMainLooper()).postDelayed(new Runnable() {
        @Override
        public void run() {
          if (deeplinkCallbackHandler == null) {
            Log.v(TAG, "SmartechBasePlugin - Deeplink handler delay is " + delay + " handler instance is " + deeplinkCallbackHandler);
            if (delay < maxDelayTime) {
              initDeeplinkCallbackHandler((delay + delayDiff), intent);
            }
          } else {
            Log.v(TAG,"SmartechBasePlugin - deeplink callback handler is available. Sending data to app.");
            deeplinkCallbackHandler.onDeeplinkIntentReceived(intent);
          }
        }
      }, delay);
    } catch (Exception e) {
      e.printStackTrace();
    }
  }
}
